@@ -61,16 +61,35 @@ MathStructure Calculator::convertToMixedUnits(const MathStructure &mstruct, cons
 		Number last_nonobsolete_nr = nr;
 		Number nr_one(1, 1);
 		Number nr_ten(10, 1);
-		while(eo.mixed_units_conversion > MIXED_UNITS_CONVERSION_DOWNWARDS && nr.isGreaterThan(nr_one)) {
+		MixedUnitsConversion muc = eo.mixed_units_conversion;
+		while(muc > MIXED_UNITS_CONVERSION_DOWNWARDS && nr.isGreaterThan(nr_one)) {
 			Unit *best_u = NULL;
 			Number best_nr;
 			int best_priority = 0;
+			if(u->subtype() == SUBTYPE_ALIAS_UNIT && ((AliasUnit*) u)->firstBaseExponent() == 1 && ((AliasUnit*) u)->firstBaseUnit()->subtype() != SUBTYPE_COMPOSITE_UNIT) {
+				size_t idiv = ((AliasUnit*) u)->expression().find("1" DIVISION);
+				if(idiv == 0 && ((AliasUnit*) u)->expression().find_first_not_of(NUMBERS, 2) == string::npos) {
+					if((((AliasUnit*) u)->mixWithBase() > 0 || (((AliasUnit*) u)->mixWithBase() == 0 && (muc == MIXED_UNITS_CONVERSION_FORCE_INTEGER || muc == MIXED_UNITS_CONVERSION_FORCE_ALL))) && (((AliasUnit*) u)->mixWithBaseMinimum() <= 1 || nr.isGreaterThanOrEqualTo(((AliasUnit*) u)->mixWithBaseMinimum()))) {
+						best_u = ((AliasUnit*) u)->firstBaseUnit();
+						MathStructure mstruct_nr(nr);
+						MathStructure m_exp(m_one);
+						((AliasUnit*) u)->convertToFirstBaseUnit(mstruct_nr, m_exp);
+						mstruct_nr.eval(eo);
+						if(!mstruct_nr.isNumber() || !m_exp.isOne() || !mstruct_nr.number().isLessThan(nr) || !mstruct_nr.number().isGreaterThanOrEqualTo(nr_one)) {
+							best_u = NULL;
+						} else {
+							best_nr = mstruct_nr.number();
+							best_priority = ((AliasUnit*) u)->mixWithBase();
+						}
+					}
+				}
+			}
 			for(size_t i = 0; i < units.size(); i++) {
 				Unit *ui = units[i];
 				if(ui->subtype() == SUBTYPE_ALIAS_UNIT && ((AliasUnit*) ui)->firstBaseUnit() == u  && ((AliasUnit*) ui)->firstBaseExponent() == 1) {
 					AliasUnit *aui = (AliasUnit*) ui;
 					int priority_i = aui->mixWithBase();
-					if(((priority_i > 0 && (!best_u || priority_i <= best_priority)) || (best_priority == 0 && priority_i == 0 && ((eo.mixed_units_conversion == MIXED_UNITS_CONVERSION_FORCE_INTEGER && aui->expression().find_first_not_of(NUMBERS) == string::npos) || eo.mixed_units_conversion == MIXED_UNITS_CONVERSION_FORCE_ALL))) && (aui->mixWithBaseMinimum() <= 1 || nr.isGreaterThanOrEqualTo(aui->mixWithBaseMinimum()))) {
+					if(((priority_i > 0 && (!best_u || priority_i <= best_priority)) || (best_priority == 0 && priority_i == 0 && ((muc == MIXED_UNITS_CONVERSION_FORCE_INTEGER && aui->expression().find_first_not_of(NUMBERS) == string::npos) || muc == MIXED_UNITS_CONVERSION_FORCE_ALL))) && (aui->mixWithBaseMinimum() <= 1 || nr.isGreaterThanOrEqualTo(aui->mixWithBaseMinimum()))) {
 						MathStructure mstruct_nr(nr);
 						MathStructure m_exp(m_one);
 						aui->convertFromFirstBaseUnit(mstruct_nr, m_exp);
@@ -84,6 +103,7 @@ MathStructure Calculator::convertToMixedUnits(const MathStructure &mstruct, cons
 				}
 			}
 			if(!best_u) break;
+			if(best_priority != 0 && muc > MIXED_UNITS_CONVERSION_DEFAULT) muc = MIXED_UNITS_CONVERSION_DEFAULT;
 			u = best_u;
 			nr = best_nr;
 			if(accept_obsolete || best_priority <= 1) {
@@ -98,31 +118,74 @@ MathStructure Calculator::convertToMixedUnits(const MathStructure &mstruct, cons
 			mstruct_new[0].set(last_nonobsolete_nr);
 			mstruct_new[1].set(u, p);
 		}
-		while(u->subtype() == SUBTYPE_ALIAS_UNIT && ((AliasUnit*) u)->firstBaseUnit()->subtype() != SUBTYPE_COMPOSITE_UNIT && ((AliasUnit*) u)->firstBaseExponent() == 1 && (((AliasUnit*) u)->mixWithBase() != 0 || eo.mixed_units_conversion == MIXED_UNITS_CONVERSION_FORCE_ALL || (eo.mixed_units_conversion == MIXED_UNITS_CONVERSION_FORCE_INTEGER && ((AliasUnit*) u)->expression().find_first_not_of(NUMBERS) == string::npos)) && !nr.isInteger() && !nr.isZero()) {
+		while((u->subtype() == SUBTYPE_BASE_UNIT || (u->subtype() == SUBTYPE_ALIAS_UNIT && ((AliasUnit*) u)->firstBaseUnit()->subtype() != SUBTYPE_COMPOSITE_UNIT && ((AliasUnit*) u)->firstBaseExponent() == 1 && (((AliasUnit*) u)->mixWithBase() != 0 || muc == MIXED_UNITS_CONVERSION_FORCE_ALL || muc == MIXED_UNITS_CONVERSION_FORCE_INTEGER))) && !nr.isInteger() && nr.isNonZero()) {
 			Number int_nr(nr);
 			int_nr.intervalToMidValue();
 			int_nr.trunc();
-			if(eo.mixed_units_conversion == MIXED_UNITS_CONVERSION_DOWNWARDS_KEEP && int_nr.isZero()) break;
+			if(muc == MIXED_UNITS_CONVERSION_DOWNWARDS_KEEP && int_nr.isZero()) break;
 			nr -= int_nr;
-			MathStructure mstruct_nr(nr);
-			MathStructure m_exp(m_one);
-			((AliasUnit*) u)->convertToFirstBaseUnit(mstruct_nr, m_exp);
-			mstruct_nr.eval(eo);
-			while(!accept_obsolete && ((AliasUnit*) u)->firstBaseUnit()->subtype() == SUBTYPE_ALIAS_UNIT && abs(((AliasUnit*) ((AliasUnit*) u)->firstBaseUnit())->mixWithBase()) > 1) {
-				u = ((AliasUnit*) u)->firstBaseUnit();
-				if(((AliasUnit*) u)->firstBaseExponent() == 1 && (((AliasUnit*) u)->mixWithBase() != 0 || eo.mixed_units_conversion == MIXED_UNITS_CONVERSION_FORCE_ALL || (eo.mixed_units_conversion == MIXED_UNITS_CONVERSION_FORCE_INTEGER && ((AliasUnit*) u)->expression().find_first_not_of(NUMBERS) == string::npos))) {
-					((AliasUnit*) u)->convertToFirstBaseUnit(mstruct_nr, m_exp);
-					mstruct_nr.eval(eo);
-					if(!mstruct_nr.isNumber() || !m_exp.isOne()) break;
-				} else {
-					mstruct_nr.setUndefined();
-					break;
+			bool b = false;
+			Number best_nr;
+			Unit *best_u = NULL;
+			bool non_int = false;
+			if(u->subtype() == SUBTYPE_ALIAS_UNIT && (muc == MIXED_UNITS_CONVERSION_FORCE_ALL || (((AliasUnit*) u)->expression().find_first_not_of(NUMBERS) == string::npos))) {
+				MathStructure mstruct_nr(nr);
+				MathStructure m_exp(m_one);
+				((AliasUnit*) u)->convertToFirstBaseUnit(mstruct_nr, m_exp);
+				mstruct_nr.eval(eo);
+				while(!accept_obsolete && ((AliasUnit*) u)->firstBaseUnit()->subtype() == SUBTYPE_ALIAS_UNIT && abs(((AliasUnit*) ((AliasUnit*) u)->firstBaseUnit())->mixWithBase()) > 1) {
+					u = ((AliasUnit*) u)->firstBaseUnit();
+					if(((AliasUnit*) u)->firstBaseExponent() == 1 && (((AliasUnit*) u)->mixWithBase() != 0 || muc == MIXED_UNITS_CONVERSION_FORCE_ALL || (muc == MIXED_UNITS_CONVERSION_FORCE_INTEGER && ((AliasUnit*) u)->expression().find_first_not_of(NUMBERS) == string::npos))) {
+						((AliasUnit*) u)->convertToFirstBaseUnit(mstruct_nr, m_exp);
+						mstruct_nr.eval(eo);
+						if(!mstruct_nr.isNumber() || !m_exp.isOne()) break;
+					} else {
+						mstruct_nr.setUndefined();
+						break;
+					}
+				}
+				if(mstruct_nr.isNumber() && m_exp.isOne()) {
+					if(mstruct_nr.number().isLessThanOrEqualTo(nr)) {
+						if(muc == MIXED_UNITS_CONVERSION_FORCE_ALL) {
+							best_u = ((AliasUnit*) u)->firstBaseUnit();
+							best_nr = mstruct_nr.number();
+							non_int = true;
+						}
+					} else {
+						if(((AliasUnit*) u)->mixWithBase() != 0 && muc > MIXED_UNITS_CONVERSION_DEFAULT) muc = MIXED_UNITS_CONVERSION_DEFAULT;
+						u = ((AliasUnit*) u)->firstBaseUnit();
+						nr = mstruct_nr.number();
+						b = true;
+					}
 				}
 			}
-			if(!mstruct_nr.isNumber() || !m_exp.isOne()) break;
-			if(eo.mixed_units_conversion == MIXED_UNITS_CONVERSION_FORCE_ALL && mstruct_nr.number().isLessThanOrEqualTo(nr)) break;
-			u = ((AliasUnit*) u)->firstBaseUnit();
-			nr = mstruct_nr.number();
+			if(!b) {
+				Number best_nr;
+				int best_priority = 0;
+				for(size_t i = 0; i < units.size(); i++) {
+					Unit *ui = units[i];
+					if(ui->subtype() == SUBTYPE_ALIAS_UNIT && ((AliasUnit*) ui)->firstBaseUnit() == u && ((AliasUnit*) ui)->firstBaseExponent() == 1) {
+						AliasUnit *aui = (AliasUnit*) ui;
+						int priority_i = aui->mixWithBase();
+						if(aui->expression().find("1" DIVISION) == 0 && aui->expression().find_first_not_of(NUMBERS DIVISION, 2) == string::npos && ((priority_i > 0 && (!best_u || priority_i <= best_priority)) || (best_priority == 0 && priority_i == 0 && (muc == MIXED_UNITS_CONVERSION_FORCE_INTEGER || muc == MIXED_UNITS_CONVERSION_FORCE_ALL)))) {
+							MathStructure mstruct_nr(nr);
+							MathStructure m_exp(m_one);
+							aui->convertFromFirstBaseUnit(mstruct_nr, m_exp);
+							mstruct_nr.eval(eo);
+							if(mstruct_nr.isNumber() && m_exp.isOne() && mstruct_nr.number().isGreaterThan(nr) && mstruct_nr.number().isGreaterThanOrEqualTo(nr_one) && (!best_u || non_int || mstruct_nr.number().isLessThan(best_nr))) {
+								non_int = false;
+								best_u = ui;
+								best_nr = mstruct_nr.number();
+								best_priority = priority_i;
+							}
+						}
+					}
+				}
+				if(!best_u) break;
+				if(best_priority != 0 && muc > MIXED_UNITS_CONVERSION_DEFAULT) muc = MIXED_UNITS_CONVERSION_DEFAULT;
+				u = best_u;
+				nr = best_nr;
+			}
 			MathStructure mstruct_term;
 			if(negated) {
 				Number pos_nr(nr);
@@ -271,6 +334,19 @@ MathStructure Calculator::convert(const MathStructure &mstruct, KnownVariable *t
 	cleanMessages(mstruct, n_messages + 1);
 	return mstruct_new;
 }
+long int count_unit_powers(const MathStructure &m) {
+	if(m.isPower() && m[0].isUnit() && m[1].isInteger()) {
+		long int exp = m[1].number().lintValue();
+		if(exp < 0) return -exp;
+		return exp;
+	}
+	if(m.isUnit()) return 1;
+	long int exp = 0;
+	for(size_t i = 0; i < m.size(); i++) {
+		exp += count_unit_powers(m[i]);
+	}
+	return exp;
+}
 MathStructure Calculator::convert(const MathStructure &mstruct, Unit *to_unit, const EvaluationOptions &eo, bool always_convert, bool convert_to_mixed_units) {
 	CompositeUnit *cu = NULL;
 	if(to_unit->subtype() == SUBTYPE_COMPOSITE_UNIT) cu = (CompositeUnit*) to_unit;
@@ -395,45 +471,84 @@ MathStructure Calculator::convert(const MathStructure &mstruct, Unit *to_unit, c
 			}
 		}
 		if(b) {
+
 			eo2.approximation = eo.approximation;
 			eo2.sync_units = true;
 			eo2.keep_prefixes = false;
-			MathStructure mbak(mstruct_new);
-			mstruct_new.divide(MathStructure(to_unit, NULL));
-			mstruct_new.eval(eo2);
-			if(mstruct_new.containsType(STRUCT_UNIT)) {
-				mbak.inverse();
-				mbak.divide(MathStructure(to_unit, NULL));
-				mbak.eval(eo2);
-				if(!mbak.containsType(STRUCT_UNIT)) mstruct_new = mbak;
-			}
 
 			bool b_eval = true;
-			if(cu) {
-				MathStructure mstruct_cu(cu->generateMathStructure(false, eo.keep_prefixes));
-				Prefix *p = NULL;
-				size_t i = 1;
-				Unit *u = cu->get(i, NULL, &p);
-				while(u) {
-					size_t i2 = i + 1;
-					if(b_eval) {
-						Unit *u2 = cu->get(i2);
-						while(u2) {
-							if(u2->baseUnit() == u->baseUnit()) {
-								b_eval = false;
-								break;
-							}
-							i2++;
-							u2 = cu->get(i2);
+			if(to_unit != priv->u_celsius && to_unit != priv->u_fahrenheit) {
+				MathStructure mbak(mstruct_new);
+				mstruct_new.divide_nocopy(new MathStructure(to_unit, NULL));
+				mstruct_new.eval(eo2);
+				size_t n = count_unit_powers(mstruct_new);
+				int exp = 1;
+				if(n > 0) {
+					MathStructure mtest(mbak);
+					mtest.inverse();
+					mtest.divide_nocopy(new MathStructure(to_unit, NULL));
+					mtest.eval(eo2);
+					if(!mtest.containsType(STRUCT_UNIT)) {
+						mstruct_new = mtest;
+					} else if(!cu || (cu->countUnits() == 1 && (cu->get(1, &exp) && exp == 1))) {
+						mtest = mbak;
+						while(exp > -10) {
+							mtest.multiply_nocopy(new MathStructure(to_unit, NULL));
+							mtest.eval(eo2);
+							size_t ntest = count_unit_powers(mtest);
+							if(ntest >= n) break;
+							n = ntest;
+							if(exp == 1) exp = -1;
+							else exp--;
+							mstruct_new = mtest;
 						}
+						if(exp == 1) {
+							mtest = mstruct_new;
+							while(exp < 10) {
+								mtest.divide_nocopy(new MathStructure(to_unit, NULL));
+								mtest.eval(eo2);
+								size_t ntest = count_unit_powers(mtest);
+								if(ntest >= n) break;
+								n = ntest;
+								exp++;
+								mstruct_new = mtest;
+							}
+						}
+					} else {
+						exp = 1;
 					}
-					mstruct_new.setPrefixForUnit(u, p);
-					i++;
-					u = cu->get(i, NULL, &p);
 				}
-				mstruct_new.multiply(mstruct_cu);
-			} else {
-				mstruct_new.multiply(MathStructure(to_unit, eo.keep_prefixes ? decimal_null_prefix : NULL));
+				if(cu) {
+					MathStructure *mstruct_cu = new MathStructure(cu->generateMathStructure(false, eo.keep_prefixes));
+					Prefix *p = NULL;
+					size_t i = 1;
+					Unit *u = cu->get(i, NULL, &p);
+					while(u) {
+						size_t i2 = i + 1;
+						if(b_eval) {
+							Unit *u2 = cu->get(i2);
+							while(u2) {
+								if(u2->baseUnit() == u->baseUnit()) {
+									b_eval = false;
+									break;
+								}
+								i2++;
+								u2 = cu->get(i2);
+							}
+						}
+						mstruct_new.setPrefixForUnit(u, p);
+						i++;
+						u = cu->get(i, NULL, &p);
+					}
+					if(exp != 1) {
+						if(mstruct_cu->isPower()) (*mstruct_cu)[1].number() *= exp;
+						else mstruct_cu->raise(exp);
+					}
+					mstruct_new.multiply_nocopy(mstruct_cu);
+				} else {
+					mstruct_new.multiply_nocopy(new MathStructure(to_unit, eo.keep_prefixes ? decimal_null_prefix : NULL));
+					if(exp != 1) mstruct_new.last().raise(exp);
+				}
 			}
 
 			eo2.sync_units = false;
