@@ -16,6 +16,10 @@
 #include "Calculator.h"
 #include "MathStructure.h"
 #include "Number.h"
+#include "BuiltinFunctions.h"
+
+using std::string;
+using std::vector;
 
 Assumptions::Assumptions() : i_type(ASSUMPTION_TYPE_NUMBER), i_sign(ASSUMPTION_SIGN_UNKNOWN), fmin(NULL), fmax(NULL), b_incl_min(true), b_incl_max(true) {}
 Assumptions::~Assumptions() {}
@@ -51,7 +55,7 @@ void Assumptions::setSign(AssumptionSign as) {
 		i_type = ASSUMPTION_TYPE_REAL;
 	}
 }
-	
+
 void Assumptions::setMin(const Number *nmin) {
 	if(!nmin) {
 		if(fmin) {
@@ -386,7 +390,7 @@ bool set_precision_of_numbers(MathStructure &mstruct, int i_prec) {
 extern bool set_uncertainty(MathStructure &mstruct, MathStructure &munc, const EvaluationOptions &eo = default_evaluation_options, bool do_eval = false);
 extern bool create_interval(MathStructure &mstruct, const MathStructure &m1, const MathStructure &m2);
 bool replace_f_interval(MathStructure &mstruct) {
-	if(mstruct.isFunction() && mstruct.function() == CALCULATOR->f_interval && mstruct.size() == 2) {
+	if(mstruct.isFunction() && mstruct.function()->id() == FUNCTION_ID_INTERVAL && mstruct.size() == 2) {
 		if(mstruct[0].isNumber() && mstruct[1].isNumber()) {
 			Number nr;
 			if(nr.setInterval(mstruct[0].number(), mstruct[1].number())) {
@@ -398,7 +402,7 @@ bool replace_f_interval(MathStructure &mstruct) {
 			MathStructure m2(mstruct[1]);
 			if(create_interval(mstruct, m1, m2)) return true;
 		}
-	} else if(mstruct.isFunction() && mstruct.function() == CALCULATOR->f_uncertainty && mstruct.size() == 3 && mstruct[2].isNumber()) {
+	} else if(mstruct.isFunction() && mstruct.function()->id() == FUNCTION_ID_UNCERTAINTY && mstruct.size() == 3 && mstruct[2].isNumber()) {
 		bool b_rel = mstruct[2].number().getBoolean();
 		if(mstruct[0].isNumber() && mstruct[1].isNumber()) {
 			Number nr(mstruct[0].number());
@@ -462,7 +466,7 @@ const MathStructure &KnownVariable::get() {
 		}
 		if(!sunit.empty() && (!CALCULATOR->variableUnitsEnabled() || sunit != "auto")) {
 			m->removeType(STRUCT_UNIT);
-			if(m->containsType(STRUCT_UNIT, false, true, true) != 0) m->transform(CALCULATOR->f_stripunits);
+			if(m->containsType(STRUCT_UNIT, false, true, true) != 0) m->transformById(FUNCTION_ID_STRIP_UNITS);
 		}
 		if(!suncertainty.empty()) {
 			Number nr_u(suncertainty);
@@ -638,3 +642,40 @@ void NowVariable::calculate(MathStructure &m) const {
 	m.set(dt);
 }
 
+#ifdef __linux__
+#	include <sys/sysinfo.h>
+#endif
+
+#if defined __linux__ || defined _WIN32
+
+#include <fstream>
+
+void UptimeVariable:: calculate(MathStructure &m) const {
+	Number nr;
+#	ifdef __linux__
+	std::ifstream proc_uptime("/proc/uptime", std::ios::in);
+	if(proc_uptime.is_open()) {
+		string s_uptime;
+		getline(proc_uptime, s_uptime, ' ');
+		nr.set(s_uptime);
+	} else {
+		struct sysinfo sf;
+		if(!sysinfo(&sf)) nr = (long int) sf.uptime;
+	}
+#	elif _WIN32
+	ULONGLONG i_uptime = GetTickCount64();
+	nr.set((long int) (i_uptime % 1000), 1000);
+	nr += (long int) (i_uptime / 1000);
+#	endif
+	m = nr;
+	Unit *u = CALCULATOR->getUnit("s");
+	if(u) m *= u;
+}
+UptimeVariable::UptimeVariable() : DynamicVariable("", "uptime") {
+	setApproximate(false);
+	always_recalculate = true;
+}
+UptimeVariable::UptimeVariable(const UptimeVariable *variable) {set(variable);}
+ExpressionItem *UptimeVariable::copy() const {return new UptimeVariable(this);}
+#endif
+
