@@ -347,7 +347,7 @@ void print_function(MathFunction *f) {
 		str += ")";
 		fprintf(ffile, "<para><command>%s</command></para>\n", str.c_str());
 		for(size_t i2 = 1; i2 <= f->countNames(); i2++) {
-			if(&f->getName(i2) != ename && !f->getName(i2).completion_only) {
+			if(&f->getName(i2) != ename) {
 				fprintf(ffile, "<para><command>%s</command></para>", f->getName(i2).name.c_str());
 			}
 		}
@@ -450,11 +450,9 @@ void print_variable(Variable *v) {
 		fprintf(vfile, "<entry><para>%s</para></entry>\n", v->title().c_str());
 		bool b_first = true;
 		for(size_t i2 = 1; i2 <= v->countNames(); i2++) {
-			if(!v->getName(i2).completion_only) {
-				if(!b_first) str += " / ";
-				b_first = false;
-				str += v->getName(i2).name;
-			}
+			if(!b_first) str += " / ";
+			b_first = false;
+			str += v->getName(i2).name;
 		}
 		fprintf(vfile, "<entry><para>%s</para></entry>\n", str.c_str());
 		value = "";
@@ -548,11 +546,9 @@ void print_unit(Unit *u) {
 		fprintf(ufile, "<entry><para>%s</para></entry>\n", u->title().c_str());
 		bool b_first = true;
 		for(size_t i2 = 1; i2 <= u->countNames(); i2++) {
-			if(!u->getName(i2).completion_only) {
-				if(!b_first) str += " / ";
-				b_first = false;
-				str += u->getName(i2).name;
-			}
+			if(!b_first) str += " / ";
+			b_first = false;
+			str += u->getName(i2).name;
 		}
 		if(u->subtype() == SUBTYPE_COMPOSITE_UNIT) {
 			fprintf(ufile, "<entry><para>(%s)</para></entry>\n", str.c_str());
